package fr.toulouse.jug.languageserver.tests;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;

import org.eclipse.lsp4j.DidOpenTextDocumentParams;
import org.eclipse.lsp4j.Hover;
import org.eclipse.lsp4j.InitializeParams;
import org.eclipse.lsp4j.MessageActionItem;
import org.eclipse.lsp4j.MessageParams;
import org.eclipse.lsp4j.Position;
import org.eclipse.lsp4j.PublishDiagnosticsParams;
import org.eclipse.lsp4j.ShowMessageRequestParams;
import org.eclipse.lsp4j.TextDocumentIdentifier;
import org.eclipse.lsp4j.TextDocumentItem;
import org.eclipse.lsp4j.TextDocumentPositionParams;
import org.eclipse.lsp4j.jsonrpc.Launcher;
import org.eclipse.lsp4j.launch.LSPLauncher;
import org.eclipse.lsp4j.services.LanguageClient;
import org.eclipse.lsp4j.services.LanguageServer;
import org.junit.Assert;
import org.junit.Test;

import fr.toulouse.jug.languageserver.Main;
import fr.toulouse.jug.languageserver.ToulouseLanguageServer;

public class TestLanguageServer {

	public void check(LanguageServer ls) throws IOException, InterruptedException, ExecutionException {
		ls.initialize(new InitializeParams());
		TextDocumentItem doc = new TextDocumentItem();
		File f = File.createTempFile("blah", ".tls");
		f.deleteOnExit();
		doc.setUri(f.toURI().toString());
		doc.setText("allo");
		ls.getTextDocumentService().didOpen(new DidOpenTextDocumentParams(doc, doc.getText()));
		
		TextDocumentIdentifier id = new TextDocumentIdentifier(doc.getUri());
		CompletableFuture<Hover> hover = ls.getTextDocumentService().hover(new TextDocumentPositionParams(id, doc.getUri(), new Position(0, 1)));
		Assert.assertEquals("allo", hover.get().getContents().get(0).getLeft());
		
		hover = ls.getTextDocumentService().hover(new TextDocumentPositionParams(id, doc.getUri(), new Position(0, 0)));
		Assert.assertEquals("allo", hover.get().getContents().get(0).getLeft());
	}

	@Test
	public void checkLocal() throws IOException, InterruptedException, ExecutionException {
		check(new ToulouseLanguageServer());
	}
	
	@Test
	public void testMain() throws Exception {
		PipedInputStream serverInput = new PipedInputStream();
		PipedOutputStream serverOutput = new PipedOutputStream();
		new Thread(() -> {
			try {
				new Main().startServer(serverInput, serverOutput);
			} catch (InterruptedException | ExecutionException e) {
				e.printStackTrace();
			}
		}).start();
		Launcher<LanguageServer> createClientLauncher = LSPLauncher.createClientLauncher(new LanguageClient() {
			
			@Override
			public void telemetryEvent(Object object) {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public CompletableFuture<MessageActionItem> showMessageRequest(ShowMessageRequestParams requestParams) {
				// TODO Auto-generated method stub
				return null;
			}
			
			@Override
			public void showMessage(MessageParams messageParams) {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public void publishDiagnostics(PublishDiagnosticsParams diagnostics) {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public void logMessage(MessageParams message) {
				// TODO Auto-generated method stub
				
			}
		}, new PipedInputStream(serverOutput), new PipedOutputStream(serverInput));
		createClientLauncher.startListening();
		LanguageServer server = createClientLauncher.getRemoteProxy();
		check(server);
	}
}
